﻿using System;
using System.Diagnostics;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using Ozeki.Media;
using Ozeki.Media.MediaHandlers;
using Ozeki.Network.Nat;
using Ozeki.VoIP;
using Ozeki.VoIP.Media;
using Ozeki.VoIP.SDK;
using Ozeki.Common;


namespace WPF_Softphone
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class PhoneWindow : Window
    {
        ISoftPhone softPhone;
        IPhoneLine phoneLine;
        PhoneLineState phoneLineInformation;
        IPhoneCall call;
        private Microphone microphone = Microphone.GetDefaultDevice();
        private Speaker speaker = Speaker.GetDefaultDevice();
        MediaConnector connector = new MediaConnector();
        PhoneCallAudioSender mediaSender = new PhoneCallAudioSender();
        PhoneCallAudioReceiver mediaReceiver = new PhoneCallAudioReceiver();
        bool inComingCall;

        public PhoneWindow()
        {
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            InitializeSoftPhone();
        }

        #region Ozeki VoIP-SIP SDK's events

        /// <summary>
        /// Occurs when phone linde state has changed.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void phoneLine_PhoneLineInformation(object sender, VoIPEventArgs<PhoneLineState> e)
        {
            phoneLineInformation = e.Item;
            InvokeGUIThread(() =>
            {
                textBlockIdentifier.Text = ((IPhoneLine)sender).SIPAccount.RegisterName;
                if (e.Item == PhoneLineState.RegistrationSucceeded)
                {
                    textBlockRegStatus.Text = "Online";
                    textBlockCallStateInfo.Text = "Registration succeeded";
                }
                else
                    textBlockCallStateInfo.Text = e.Item.ToString();
            });
        }

        /// <summary>
        /// Occurs when an incoming call request has received.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void softPhone_IncomingCall(object sender, VoIPEventArgs<IPhoneCall> e)
        {
            InvokeGUIThread(() =>
            {
                textBlockCallStateInfo.Text = String.Format("Incoming call from {0}", e.Item.DialInfo.DisplayName);
                call = e.Item;
                WireUpCallEvents(call);
                inComingCall = true;
            });
        }

        /// <summary>
        /// Occurs when the phone call state has changed.
        /// Example: Ringing, Incall, Completed, Cancelled ...
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void call_CallStateChanged(object sender, VoIPEventArgs<CallState> e)
        {
            InvokeGUIThread(() => { textBlockCallStateInfo.Text = e.Item.ToString(); });

            switch (e.Item)
            {
                case CallState.InCall:
                    microphone.Start();
                    connector.Connect(microphone, mediaSender);

                    speaker.Start();
                    connector.Connect(mediaReceiver, speaker);
                    
                    mediaSender.AttachToCall(call);
                    mediaReceiver.AttachToCall(call);
                    break;
                case CallState.Completed:
                    microphone.Stop();
                    connector.Disconnect(microphone, mediaSender);

                    speaker.Stop();
                    connector.Disconnect(mediaReceiver, speaker);

                    mediaSender.Detach();
                    mediaReceiver.Detach();

                    WireDownCallEvents(sender as IPhoneCall);
                    call = null;
                    
                    InvokeGUIThread(() => { textBlockDialingNumber.Text = string.Empty; });
                    break;
                case CallState.Cancelled:
                    WireDownCallEvents(sender as IPhoneCall);
                    call = null; 
                    break;
            }
        }

        /// <summary>
        /// Display DTMF signals.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void call_DtmfReceived(object sender, VoIPEventArgs<OzTuple<VoIPMediaType, DtmfSignal>> e)
        {
            InvokeGUIThread(() =>
            {
                textBlockCallStateInfo.Text = String.Format("DTMF signal received: {0} ", e.Item.Item2.Signal);
            });
        }

        /// <summary>
        /// There are certain situations when the call cannot be created, for example the dialed number is not available 
        /// or maybe there is no endpoint to the dialed PBX, or simply the telephone line is busy. 
        /// This event handling is for displaying these events.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void call_CallErrorOccured(object sender, VoIPEventArgs<CallError> e)
        {
            InvokeGUIThread(() =>
            {
                textBlockCallStateInfo.Text = e.Item.ToString();
            });
        }

        #endregion

        #region Helper Functions
        /// <summary>
        /// It initializes a softphone object with a SIP BPX, and it is for requisiting a SIP account that is nedded for a SIP PBX service. It registers this SIP
        /// account to the SIP PBX through an ’IphoneLine’ object which represents the telephoneline. 
        /// If the telephone registration is successful we have a call ready softphone. In this example the softphone can be reached by dialing the number 891.
        /// </summary>
        private void InitializeSoftPhone()
        {
            try
            {
                softPhone = SoftPhoneFactory.CreateSoftPhone(SoftPhoneFactory.GetLocalIP(), 5700, 5750, 5700);
                softPhone.IncomingCall += new EventHandler<VoIPEventArgs<IPhoneCall>>(softPhone_IncomingCall);
                phoneLine = softPhone.CreatePhoneLine(new SIPAccount(true, "oz875", "oz875", "oz875", "oz875", "192.168.112.100", 5060),new NatConfiguration(NatTraversalMethod.None,""));
                phoneLine.PhoneLineStateChanged += new EventHandler<VoIPEventArgs<PhoneLineState>>(phoneLine_PhoneLineInformation);

                softPhone.RegisterPhoneLine(phoneLine);
            }

            catch (Exception ex)
            {
                MessageBox.Show(
                    String.Format("You didn't give your local IP adress, so the program won't run properly.\n {0}",
                                  ex.Message), "Ozeki WPF Softphone Sample", MessageBoxButton.OK, MessageBoxImage.Error);

				var sb = new StringBuilder();
                sb.AppendLine("Some error happened.");
                sb.AppendLine();
                sb.AppendLine("Exception:");
                sb.AppendLine(ex.Message);
                sb.AppendLine();
                if(ex.InnerException != null)
                {
                    sb.AppendLine("Inner Exception:");
                    sb.AppendLine(ex.InnerException.Message);
                    sb.AppendLine();
                }
                sb.AppendLine("StackTrace:");
                sb.AppendLine(ex.StackTrace);

                MessageBox.Show(sb.ToString());
            }
        }

        /// <summary>
        /// It signs up to the necessary events of a call transact.
        /// </summary>
        private void WireUpCallEvents(IPhoneCall call)
        {
            call.CallStateChanged += new EventHandler<VoIPEventArgs<CallState>>(call_CallStateChanged);
            call.DtmfReceived += new EventHandler<VoIPEventArgs<OzTuple<VoIPMediaType, DtmfSignal>>>(call_DtmfReceived);
            call.CallErrorOccured += new EventHandler<VoIPEventArgs<CallError>>(call_CallErrorOccured);
        }

        /// <summary>
        /// It signs down from the necessary events of a call transact.
        /// </summary>
        private void WireDownCallEvents(IPhoneCall call)
        {
            if (call != null)
            {
                call.CallStateChanged -= new EventHandler<VoIPEventArgs<CallState>>(call_CallStateChanged);
                call.DtmfReceived -= new EventHandler<VoIPEventArgs<OzTuple<VoIPMediaType, DtmfSignal>>>(call_DtmfReceived);
                call.CallErrorOccured -= new EventHandler<VoIPEventArgs<CallError>>(call_CallErrorOccured);
            }
        }

        /// <summary>
        /// The controls of the Windows Form applications can only be modified on the GUI thread. This method grants access to the GUI thread.
        /// </summary>
        /// <param name="action"></param>
        private void InvokeGUIThread(Action action)
        {
            Dispatcher.Invoke(action);
        }
        #endregion

        #region Keypad events

        /// <summary>
        /// It starts a call with the dialed number or in case of an incoming call it accepts, picks up the call.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonPickUp_Click(object sender, RoutedEventArgs e)
        {
            if (inComingCall)
            {
                inComingCall = false;
                call.Accept();
                return;
            }

            if (call != null)
                return;

            if (string.IsNullOrEmpty(textBlockDialingNumber.Text))
                return;

            if (phoneLineInformation != PhoneLineState.RegistrationSucceeded && phoneLineInformation != PhoneLineState.NoRegNeeded)
            {
                MessageBox.Show("Phone line state is not valid!");
                return;
            }
            
            call = softPhone.CreateCallObject(phoneLine, textBlockDialingNumber.Text);
            WireUpCallEvents(call);
            call.Start();
        }

        /// <summary>
        /// In case of a call is in progress, it breaks the call.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonHangUp_Click(object sender, RoutedEventArgs e)
        {
            if (call != null)
            {
                if (inComingCall && call.CallState == CallState.Ringing)
                    call.Reject();
                else
                    call.HangUp();
                call = null;

                inComingCall = false; 
            }
            textBlockDialingNumber.Text = string.Empty;
        }

        /// <summary>
        /// If there is a call in progress it sends a DTMF signal according to the RFC 2833 standard else it makes the dialing number.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonKeyPadButton_Click(object sender, RoutedEventArgs e)
        {
            var btn = sender as Button;
            if (btn != null)
            {
                textBlockDialingNumber.Text += btn.Content.ToString().Trim();

                if (call != null && call.CallState == CallState.InCall)
                {   
                        int id;

                        if (btn.Tag != null && int.TryParse(btn.Tag.ToString(), out id))
                        {
                            call.StartDTMFSignal(VoIPMediaType.Audio, (DtmfNamedEvents)id);

                            call.StopDTMFSignal(VoIPMediaType.Audio, (DtmfNamedEvents)id);
                        }  
                }
            }
            e.Handled = false;
        }      

        #endregion

        #region Menu events

        private void ExitMenuItem_Click(object sender, RoutedEventArgs e)
        {
            App.Current.Shutdown();
        }

        private void HelpMenuItem_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                Process.Start("http://www.voip-sip-sdk.com/");
            }
            catch (Exception exception)
            {
                Debug.WriteLine(exception);
            }
        }

        /// <summary>
        /// Displays the "About" window.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void AboutMenuItem_Click(object sender, RoutedEventArgs e)
        {
            AboutWindow wnd = new AboutWindow();
            wnd.Owner = this;
            wnd.ShowDialog();
        }

        private void Window_MouseDown(object sender, MouseButtonEventArgs e)
        {
            DragMove();
        }
        #endregion

        /// <summary>
        /// It stops the softphone when the program is closed.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Window_Closed(object sender, EventArgs e)
        {
            softPhone.Close();
        }
    }
}
